from pdocs._version import __version__

ascii_art = rf"""

                      88
                      88
                      88
8b,dPPYba,    ,adPPYb,88   ,adPPYba,    ,adPPYba,  ,adPPYba,
88P'    "8a  a8"    `Y88  a8"     "8a  a8"     ""  I8[    ""
88       d8  8b       88  8b       d8  8b           `"Y8ba,
88b,   ,a8"  "8a,   ,d88  "8a,   ,a8"  "8a,   ,aa  aa    ]8I
88`YbbdP"'    `"8bbdP"Y8   `"YbbdP"'    `"Ybbd8"'  `"YbbdP"'
88
88        - Documentation Powered by Your Python Code -

Version: {__version__}
Copyright Timothy Edmund Crosley 2019 MIT License
"""

__doc__ = f"""
```python
{ascii_art}
```
"""



import importlib
import os
import pkgutil
import typing

import pdocs.doc


class ExtractError(Exception):
    pass


def split_module_spec(spec: str) -> typing.Tuple[str, str]:
    """
        Splits a module specification into a base path (which may be empty), and a module name.

        Raises ExtactError if the spec is invalid.
    """
    if not spec:
        raise ExtractError("Empty module spec.")
    if (os.sep in spec) or (os.altsep and os.altsep in spec):
        dirname, fname = os.path.split(spec)
        if fname.endswith(".py"):
            mname, _ = os.path.splitext(fname)
            return dirname, mname
        else:
            if "." in fname:
                raise ExtractError(
                    "Invalid module name {fname}. Mixing path and module specifications "
                    "is not supported.".format(fname=fname)
                )
            return dirname, fname
    else:
        return "", spec


def load_module(basedir: str, module: str) -> typing.Tuple[typing.Any, bool]:
    """
        Returns a module object, and whether the module is a package or not.
    """
    ispackage = False
    if basedir:
        mods = module.split(".")
        dirname = os.path.join(basedir, *mods[:-1])
        modname = mods[-1]

        pkgloc = os.path.join(dirname, modname, "__init__.py")
        fileloc = os.path.join(dirname, modname + ".py")

        if os.path.exists(pkgloc):
            location, ispackage = pkgloc, True
        elif os.path.exists(fileloc):
            location, ispackage = fileloc, False
        else:
            raise ExtractError(
                "Module {module} not found in {basedir}".format(module=module, basedir=basedir)
            )

        ispec = importlib.util.spec_from_file_location(modname, location)
        mobj = importlib.util.module_from_spec(ispec)
        try:
            # This can literally raise anything
            ispec.loader.exec_module(mobj)  # type: ignore
        except Exception as e:
            raise ExtractError("Error importing {location}: {e}".format(location=location, e=e))
        return mobj, ispackage
    else:
        try:
            # This can literally raise anything
            m = importlib.import_module(module)
        except ImportError:
            raise ExtractError("Module not found: {module}".format(module=module))
        except Exception as e:
            raise ExtractError("Error importing {module}: {e}".format(module=module, e=e))
        # This is the only case where we actually have to test whether we're a package
        if getattr(m, "__package__", False) and getattr(m, "__path__", False):
            ispackage = True
        return m, ispackage


def submodules(dname: typing.Optional[str], mname: str) -> typing.Sequence[str]:
    """
        Return a list of submodule names using a file path or import based name

        If dname is None or empty string, mname will be used as import name.
        Otherwise, the relative directory path at dname will be joined to
        package name mname, and used as the base path for searching.
    """
    if dname:
        return _submodules_from_pathing(dname, mname)
    else:
        return _submodules_from_import_name(mname)


def _submodules_from_import_name(mname: str) -> typing.Sequence[str]:
    """
        Return a list of fully qualified submodules within a package

        mname is an import based module name
    """
    loc = importlib.util.find_spec(mname).submodule_search_locations
    if loc is None:
        # Case of mname corresponding to a terminal module, and not a package
        # iter_modules returns everything it can find anywhere if loc is None,
        # which is not what we want
        return []
    as_imported = importlib.import_module(mname)
    if getattr(as_imported, "__path__", None):
        [loc.append(path) for path in as_imported.__path__ if path not in loc]
    ret = []
    for mi in pkgutil.iter_modules(loc, prefix=mname + "."):
        if isinstance(mi, tuple):
            # Python 3.5 compat
            ret.append(mi[1])
        else:
            ret.append(mi.name)
    ret.sort()
    return ret


def _submodules_from_pathing(dname: str, mname: str) -> typing.Sequence[str]:
    """
        Return a list of fully qualified submodules within a package, given a
        base directory and a fully qualified module name.

        dname is a directory file path, under which mname is stored,
        and mname is module to search for submodules from
    """
    loc = os.path.join(dname, *mname.split("."))
    ret = []
    for mi in pkgutil.iter_modules([loc], prefix=mname + "."):
        if isinstance(mi, tuple):
            # Python 3.5 compat
            ret.append(mi[1])
        else:
            ret.append(mi.name)
    ret.sort()
    return ret


def _extract_module(dname: str, mname: str, parent=None) -> typing.Any:
    m, pkg = load_module(dname, mname)
    mod = pdocs.doc.Module(mname, m, parent)
    if pkg:
        for submodule_full_name in submodules(dname, mname):
            if submodule_full_name.split(".")[-1].startswith("_"):
                continue

            mod.submodules.append(_extract_module(dname, submodule_full_name, parent=mod))
    return mod


def extract_module(spec: str):
    """
        Extracts and returns a module object. The spec argument can have the
        following forms:

        Simple module: "foo.bar"
        Module path: "./path/to/module"
        File path: "./path/to/file.py"

        This function always invalidates caches to enable hot load and reload.

        May raise ExtactError.
    """
    importlib.invalidate_caches()
    dname, mname = split_module_spec(spec)
    return _extract_module(dname, mname)



import ast
import inspect
import typing

import docstring_parser

__pdoc__ = {}


def _source(obj):
    """
    Returns the source code of the Python object `obj` as a list of
    lines. This tries to extract the source from the special
    `__wrapped__` attribute if it exists. Otherwise, it falls back
    to `inspect.getsourcelines`.

    If neither works, then the empty list is returned.
    """
    try:
        return inspect.getsourcelines(obj.__wrapped__)[0]
    except BaseException:
        pass
    try:
        return inspect.getsourcelines(obj)[0]
    except Exception:
        return []


def _var_docstrings(tree, module, cls=None, init=False):
    """
    Extracts variable docstrings given `tree` as the abstract syntax,
    `module` as a `pdoc.Module` containing `tree` and an option `cls`
    as a `pdoc.Class` corresponding to the tree. In particular, `cls`
    should be specified when extracting docstrings from a class or an
    `__init__` method. Finally, `init` should be `True` when searching
    the AST of an `__init__` method so that `_var_docstrings` will only
    accept variables starting with `self.` as instance variables.

    A dictionary mapping variable name to a `pdoc.Variable` object is
    returned.
    """
    vs = {}
    children = list(ast.iter_child_nodes(tree))
    for i, child in enumerate(children):
        if isinstance(child, ast.Assign) and len(child.targets) == 1:
            if not init and isinstance(child.targets[0], ast.Name):
                name = child.targets[0].id
            elif (
                isinstance(child.targets[0], ast.Attribute)
                and isinstance(child.targets[0].value, ast.Name)
                and child.targets[0].value.id == "self"
            ):
                name = child.targets[0].attr
            else:
                continue
            if not _is_exported(name) and name not in getattr(module, "__all__", []):
                continue

            docstring = ""
            if (
                i + 1 < len(children)
                and isinstance(children[i + 1], ast.Expr)
                and isinstance(children[i + 1].value, ast.Str)
            ):
                docstring = children[i + 1].value.s

            vs[name] = Variable(name, module, docstring, cls=cls)
    return vs


def _is_exported(ident_name):
    """
    Returns `True` if `ident_name` matches the export criteria for an
    identifier name.

    This should not be used by clients. Instead, use
    `pdoc.Module.is_public`.
    """
    return not ident_name.startswith("_")


def _is_method(cls: typing.Type, method_name: str) -> bool:
    """
    Returns `True` if the given method is a regular method,
    i.e. it's neither annotated with @classmethod nor @staticmethod.
    """
    func = getattr(cls, method_name, None)
    if inspect.ismethod(func):
        # If the function is already bound, it's a classmethod.
        # Regular methods are not bound before initialization.
        return False
    for c in inspect.getmro(cls):
        if method_name in c.__dict__:
            return not isinstance(c.__dict__[method_name], staticmethod)
    else:
        raise ValueError(
            "{method_name} not found in {cls}.".format(method_name=method_name, cls=cls)
        )


def _filter(items, kind, attributes_set=(), attributes_not_set=(), sort=True):
    items = (item for item in items if isinstance(item, kind))
    for attribute_set in attributes_set:
        items = (item for item in items if getattr(item, attribute_set, False))
    for attribute_not_set in attributes_not_set:
        items = (item for item in items if not getattr(item, attribute_not_set, False))
    if sort:
        return sorted(items)
    else:
        return tuple(items)


class Doc(object):
    """
    A base class for all documentation objects.

    A documentation object corresponds to *something* in a Python module
    that has a docstring associated with it. Typically, this only includes
    modules, classes, functions and methods. However, `pdoc` adds support
    for extracting docstrings from the abstract syntax tree, which means
    that variables (module, class or instance) are supported too.

    A special type of documentation object `pdoc.External` is used to
    represent identifiers that are not part of the public interface of
    a module. (The name "External" is a bit of a misnomer, since it can
    also correspond to unexported members of the module, particularly in
    a class's ancestor list.)
    """

    def __init__(self, name, module, docstring):
        """
        Initializes a documentation object, where `name` is the public
        identifier name, `module` is a `pdoc.Module` object, and
        `docstring` is a string containing the docstring for `name`.
        """
        self.module = module
        """
        The module documentation object that this object was defined
        in.
        """

        self.name = name
        """
        The identifier name for this object.
        """

        self.docstring = inspect.cleandoc(docstring or "")
        """
        The docstring for this object. It has already been cleaned
        by `inspect.cleandoc`.
        """

        try:
            self.parsed_docstring = docstring_parser.parse(self.docstring)
        except (docstring_parser.ParseError, ValueError):
            self.parsed_docstring = None
        """
        The parsed docstring for this object.
        """

    @property
    def source(self):
        """
        Returns the source code of the Python object `obj` as a list of
        lines. This tries to extract the source from the special
        `__wrapped__` attribute if it exists. Otherwise, it falls back
        to `inspect.getsourcelines`.

        If neither works, then the empty list is returned.
        """
        raise NotImplementedError("source() method should be implemented by sub casses")

    @property
    def refname(self):
        """
        Returns an appropriate reference name for this documentation
        object. Usually this is its fully qualified path. Every
        documentation object must provide this property.

        e.g., The refname for this property is
        <code>pdoc.Doc.refname</code>.
        """
        raise NotImplementedError("refname() method should be implemented by sub casses")

    def __lt__(self, other):
        return self.name < other.name

    def is_empty(self):
        """
        Returns true if the docstring for this object is empty.
        """
        return len(self.docstring.strip()) == 0


class Module(Doc):
    """
    Representation of a module's documentation.
    """

    __pdoc__["Module.module"] = "The Python module object."
    __pdoc__[
        "Module.name"
    ] = """
        The name of this module with respect to the context in which
        it was imported. It is always an absolute import path.
        """

    def __init__(self, name, module, parent):
        """
        Creates a `Module` documentation object given the actual
        module Python object.
        """
        super().__init__(name, module, inspect.getdoc(module))
        self.parent = parent

        self.doc = {}
        """A mapping from identifier name to a documentation object."""

        self.refdoc = {}
        """
        The same as `pdoc.Module.doc`, but maps fully qualified
        identifier names to documentation objects.
        """

        self.submodules = []

        vardocs = {}
        try:
            tree = ast.parse(inspect.getsource(self.module))
            vardocs = _var_docstrings(tree, self, cls=None)
        except BaseException:
            pass
        self._declared_variables = vardocs.keys()

        public = self.__public_objs()
        for name, obj in public.items():
            # Skip any identifiers that already have doco.
            if name in self.doc and not self.doc[name].is_empty():
                continue

            # Functions and some weird builtins?, plus methods, classes,
            # modules and module level variables.
            if inspect.isroutine(obj):
                self.doc[name] = Function(name, self, obj)
            elif inspect.isclass(obj):
                self.doc[name] = Class(name, self, obj)
            elif name in vardocs:
                self.doc[name] = vardocs[name]
            else:
                # Catch all for variables.
                self.doc[name] = Variable(name, self, "", cls=None)

        # Now see if we can grab inheritance relationships between classes.
        for docobj in self.doc.values():
            if isinstance(docobj, Class):
                docobj._fill_inheritance()

        # Build the reference name dictionary.
        for _basename, docobj in self.doc.items():
            self.refdoc[docobj.refname] = docobj
            if isinstance(docobj, Class):
                for v in docobj.class_variables():
                    self.refdoc[v.refname] = v
                for v in docobj.instance_variables():
                    self.refdoc[v.refname] = v
                for f in docobj.methods():
                    self.refdoc[f.refname] = f
                for f in docobj.functions():
                    self.refdoc[f.refname] = f

        # Finally look for more docstrings in the __pdoc__ override.
        for name, docstring in getattr(self.module, "__pdoc__", {}).items():
            refname = "%s.%s" % (self.refname, name)
            if docstring is None:
                self.doc.pop(name, None)
                self.refdoc.pop(refname, None)
                continue

            dobj = self.find_ident(refname)
            if isinstance(dobj, External):
                continue
            dobj.docstring = inspect.cleandoc(docstring)
            try:
                dobj.parsed_docstring = docstring_parser.parse(dobj.docstring)
            except docstring_parser.ParseError:
                dobj.parsed_docstring = None

    @property
    def source(self):
        return _source(self.module)

    @property
    def refname(self):
        return self.name

    @property
    def is_namespace(self):
        """Returns `True` if this module represents a
           [namespace package](https://packaging.python.org/guides/packaging-namespace-packages/).
        """
        return self.module.__spec__.origin in (None, "namespace")

    def mro(self, cls):
        """
        Returns a method resolution list of ancestor documentation objects
        for `cls`, which must be a documentation object.

        The list will contain objects belonging to `pdoc.Class` or
        `pdoc.External`. Objects belonging to the former are exported
        classes either in this module or in one of its sub-modules.
        """
        return [self.find_class(c) for c in inspect.getmro(cls.cls) if c not in (cls.cls, object)]

    def descendents(self, cls):
        """
        Returns a descendent list of documentation objects for `cls`,
        which must be a documentation object.

        The list will contain objects belonging to `pdoc.Class` or
        `pdoc.External`. Objects belonging to the former are exported
        classes either in this module or in one of its sub-modules.
        """
        if cls.cls == type or not hasattr(cls.cls, "__subclasses__"):
            # Is this right?
            return []

        downs = cls.cls.__subclasses__()
        return list(map(lambda c: self.find_class(c), downs))

    def is_public(self, name):
        """
        Returns `True` if and only if an identifier with name `name` is
        part of the public interface of this module. While the names
        of sub-modules are included, identifiers only exported by
        sub-modules are not checked.

        `name` should be a fully qualified name, e.g.,
        <code>pdoc.Module.is_public</code>.
        """
        return name in self.refdoc

    def find_class(self, cls):
        """
        Given a Python `cls` object, try to find it in this module
        or in any of the exported identifiers of the submodules.
        """
        for doc_cls in self.classes():
            if cls is doc_cls.cls:
                return doc_cls
        for module in self.submodules:
            doc_cls = module.find_class(cls)
            if not isinstance(doc_cls, External):
                return doc_cls
        return External("%s.%s" % (cls.__module__, cls.__name__))

    def find_ident(self, name, _seen=None):
        """
        Searches this module and **all** of its sub/super-modules for an
        identifier with name `name` in its list of exported
        identifiers according to `pdoc`. Note that unexported
        sub-modules are searched.

        A bare identifier (without `.` separators) will only be checked
        for in this module.

        The documentation object corresponding to the identifier is
        returned. If one cannot be found, then an instance of
        `External` is returned populated with the given identifier.
        """
        _seen = _seen or set()
        if self in _seen:
            return None
        _seen.add(self)

        if name == self.refname:
            return self
        if name in self.refdoc:
            return self.refdoc[name]
        for module in self.submodules:
            o = module.find_ident(name, _seen=_seen)
            if not isinstance(o, (External, type(None))):
                return o
        # Traverse also up-level super-modules
        module = self.parent
        while module is not None:
            o = module.find_ident(name, _seen=_seen)
            if not isinstance(o, (External, type(None))):
                return o
            module = module.parent
        return External(name)

    def variables(self):
        """
        Returns all documented module level variables in the module
        sorted alphabetically as a list of `pdoc.Variable`.
        """
        return _filter(self.doc.values(), Variable)

    def classes(self):
        """
        Returns all documented module level classes in the module
        sorted alphabetically as a list of `pdoc.Class`.
        """
        return _filter(self.doc.values(), Class)

    def functions(self):
        """
        Returns all documented module level functions in the module
        sorted alphabetically as a list of `pdoc.Function`.
        """
        return _filter(self.doc.values(), Function)

    def __is_exported(self, name, module):
        """
        Returns `True` if and only if `pdoc` considers `name` to be
        a public identifier for this module where `name` was defined
        in the Python module `module`.

        If this module has an `__all__` attribute, then `name` is
        considered to be exported if and only if it is a member of
        this module's `__all__` list.

        If `__all__` is not set, then whether `name` is exported or
        not is heuristically determined. Firstly, if `name` starts
        with an underscore, it will not be considered exported.
        Secondly, if `name` was defined in a module other than this
        one, it will not be considered exported. In all other cases,
        `name` will be considered exported.
        """
        if hasattr(self.module, "__all__"):
            return name in self.module.__all__
        if not _is_exported(name):
            return False
        if module is not None and self.module.__name__ != module.__name__:
            return name in self._declared_variables
        return True

    def __public_objs(self):
        """
        Returns a dictionary mapping a public identifier name to a
        Python object.
        """
        members = dict(inspect.getmembers(self.module))
        return dict(
            [
                (name, obj)
                for name, obj in members.items()
                if self.__is_exported(name, inspect.getmodule(obj))
            ]
        )

    def allmodules(self):
        yield self
        for i in self.submodules:
            yield from i.allmodules()

    def toroot(self):
        n = self
        while n:
            yield n
            n = n.parent


class Class(Doc):
    """
    Representation of a class's documentation.
    """

    def __init__(self, name, module, class_obj):
        """
        Same as `pdocs.Doc.__init__`, except `class_obj` must be a
        Python class object. The docstring is gathered automatically.
        """
        super().__init__(name, module, inspect.getdoc(class_obj))

        self.cls = class_obj
        """The class Python object."""

        self.doc = {}
        """A mapping from identifier name to a `pdoc.Doc` objects."""

        self.doc_init = {}
        """
        A special version of `pdoc.Class.doc` that contains
        documentation for instance variables found in the `__init__`
        method.
        """

        public = self.__public_objs()
        try:
            # First try and find docstrings for class variables.
            # Then move on to finding docstrings for instance variables.
            # This must be optional, since not all modules have source
            # code available.
            cls_ast = ast.parse(inspect.getsource(self.cls)).body[0]
            self.doc = _var_docstrings(cls_ast, self.module, cls=self)

            for n in cls_ast.body if "__init__" in public else []:
                if isinstance(n, ast.FunctionDef) and n.name == "__init__":
                    self.doc_init = _var_docstrings(n, self.module, cls=self, init=True)
                    break
        except BaseException:
            pass

        # Convert the public Python objects to documentation objects.
        for name, obj in public.items():
            # Skip any identifiers that already have doco.
            if name in self.doc and not self.doc[name].is_empty():
                continue
            if name in self.doc_init:
                # Let instance members override class members.
                continue

            if inspect.isroutine(obj):
                self.doc[name] = Function(
                    name, self.module, obj, cls=self, method=_is_method(self.cls, name)
                )
            elif isinstance(obj, property):
                docstring = getattr(obj, "__doc__", "")
                self.doc_init[name] = Variable(name, self.module, docstring, cls=self)
            elif not inspect.isbuiltin(obj) and not inspect.isroutine(obj):
                if name in getattr(self.cls, "__slots__", []):
                    self.doc_init[name] = Variable(name, self.module, "", cls=self)
                else:
                    self.doc[name] = Variable(name, self.module, "", cls=self)

    @property
    def source(self):
        return _source(self.cls)

    @property
    def refname(self):
        return "%s.%s" % (self.module.refname, self.cls.__name__)

    def class_variables(self):
        """
        Returns all documented class variables in the class, sorted
        alphabetically as a list of `pdoc.Variable`.
        """
        return _filter(self.doc.values(), Variable)

    def instance_variables(self):
        """
        Returns all instance variables in the class, sorted
        alphabetically as a list of `pdoc.Variable`. Instance variables
        are attributes of `self` defined in a class's `__init__`
        method.
        """
        return _filter(self.doc_init.values(), Variable)

    def methods(self):
        """
        Returns all documented methods as `pdoc.Function` objects in
        the class, sorted alphabetically.

        Unfortunately, this also includes class methods.
        """
        return _filter(self.doc.values(), Function, attributes_set=("method",))

    def functions(self):
        """
        Returns all documented static functions as `pdoc.Function`
        objects in the class, sorted alphabetically.
        """
        return _filter(self.doc.values(), Function, attributes_not_set=("method",))

    def params(self):
        """Returns back the parameters for the classes __init__ method"""
        params = Function._params(self.cls.__init__)
        return params[1:] if params[0] == "self" else params

    def _fill_inheritance(self):
        """
        Traverses this class's ancestor list and attempts to fill in
        missing documentation from its ancestor's documentation.

        The first pass connects variables, methods and functions with
        their inherited couterparts. (The templates will decide how to
        display docstrings.) The second pass attempts to add instance
        variables to this class that were only explicitly declared in
        a parent class. This second pass is necessary since instance
        variables are only discoverable by traversing the abstract
        syntax tree.
        """
        mro = [c for c in self.module.mro(self) if c != self and isinstance(c, Class)]

        def search(d, fdoc):
            for c in mro:
                doc = fdoc(c)
                if d.name in doc and isinstance(d, type(doc[d.name])):
                    return doc[d.name]
            return None

        for fdoc in (lambda c: c.doc_init, lambda c: c.doc):
            for d in fdoc(self).values():
                dinherit = search(d, fdoc)
                if dinherit is not None:
                    d.inherits = dinherit

        # Since instance variables aren't part of a class's members,
        # we need to manually deduce inheritance. Oh lawdy.
        for c in mro:
            for name in filter(lambda n: n not in self.doc_init, c.doc_init):
                d = c.doc_init[name]
                self.doc_init[name] = Variable(d.name, d.module, "", cls=self)
                self.doc_init[name].inherits = d

    def mro(self):
        """Returns back the Method Resolution Order (MRO) for this class"""
        return [
            self.module.find_class(cls)
            for cls in inspect.getmro(self.cls)
            if cls not in (self.cls, object, self)
        ]

    def subclasses(self):
        """Returns back all subclasses of this class"""
        return [self.module.find_class(cls) for cls in type.__subclasses__(self.cls)]

    def __public_objs(self):
        """
        Returns a dictionary mapping a public identifier name to a
        Python object. This counts the `__init__` method as being
        public.
        """
        _pdoc = getattr(self.module.module, "__pdoc__", {})

        def forced_out(name):
            return _pdoc.get("%s.%s" % (self.name, name), False) is None

        def exported(name):
            if _is_exported(name) and not forced_out(name):
                return name

        idents = dict(inspect.getmembers(self.cls))
        return dict([(n, o) for n, o in idents.items() if exported(n)])


class Function(Doc):
    """
    Representation of documentation for a Python function or method.
    """

    def __init__(self, name, module, func_obj, cls=None, method=False):
        """
        Same as `pdoc.Doc.__init__`, except `func_obj` must be a
        Python function object. The docstring is gathered automatically.

        `cls` should be set when this is a method or a static function
        beloing to a class. `cls` should be a `pdoc.Class` object.

        `method` should be `True` when the function is a method. In
        all other cases, it should be `False`.
        """
        super().__init__(name, module, inspect.getdoc(func_obj))

        self.func = func_obj
        """The Python function object."""

        self.cls = cls
        """
        The `pdoc.Class` documentation object if this is a method. If
        not, this is None.
        """

        self.method = method
        """
        Whether this function is a method or not.

        In particular, static class methods have this set to False.
        """

    @property
    def source(self):
        return _source(self.func)

    @property
    def refname(self):
        if self.cls is None:
            return "%s.%s" % (self.module.refname, self.name)
        else:
            return "%s.%s" % (self.cls.refname, self.name)

    def funcdef(self):
        """
        Generates the string of keywords used to define the function, for
        example `def` or `async def`.
        """
        keywords = []

        if self._is_async():
            keywords.append("async")

        keywords.append("def")

        return " ".join(keywords)

    def _is_async(self):
        """
        Returns whether is function is asynchronous, either as a coroutine or an
        async generator.
        """
        try:
            # Both of these are required because coroutines aren't classified as
            # async generators and vice versa.
            return inspect.iscoroutinefunction(self.func) or inspect.isasyncgenfunction(self.func)
        except AttributeError:
            return False

    def spec(self):
        """
        Returns a nicely formatted spec of the function's parameter
        list as a string. This includes argument lists, keyword
        arguments and default values.
        """
        return ", ".join(self.params())

    @staticmethod
    def _signature(function):
        try:
            return inspect.signature(function)
        except (TypeError, ValueError):  # We can't get a Python signature (likely C function)
            return False

    def return_annotation(self):
        """Returns back return type annotation if a valid one is found"""
        signature = self._signature(self.func)
        if not signature or signature.return_annotation == inspect._empty:
            return ""

        return inspect.formatannotation(signature.return_annotation)

    def params(self):
        """
        Returns a list where each element is a nicely formatted
        parameter of this function. This includes argument lists,
        keyword arguments and default values.
        """
        return self._params(self.func)

    @classmethod
    def _params(cls, function):
        """
        Returns a list where each element is a nicely formatted
        parameter of this function. This includes argument lists,
        keyword arguments and default values.
        """
        signature = cls._signature(function)
        if not signature:
            return ["..."]

        # The following is taken almost verbatim from the Python stdlib
        # https://github.com/python/cpython/blob/3.6/Lib/inspect.py#L3017
        #
        # This is done simply because it is hard to unstringify the result since commas could
        # be present beyond just between parameters.
        params = []
        render_pos_only_separator = False
        render_kw_only_separator = True
        for param in signature.parameters.values():

            kind = param.kind
            if kind == inspect._POSITIONAL_ONLY:
                render_pos_only_separator = True
            elif render_pos_only_separator:
                params.append("/")
                render_pos_only_separator = False

            if kind == inspect._VAR_POSITIONAL:
                render_kw_only_separator = False
            elif kind == inspect._KEYWORD_ONLY and render_kw_only_separator:
                params.append("*")
                render_kw_only_separator = False

            params.append(str(param))

        if render_pos_only_separator:
            params.append("/")

        return params

    def __lt__(self, other):
        # Push __init__ to the top.
        if "__init__" in (self.name, other.name):
            return self.name != other.name and self.name == "__init__"
        else:
            return self.name < other.name


class Variable(Doc):
    """
    Representation of a variable's documentation. This includes
    module, class and instance variables.
    """

    def __init__(self, name, module, docstring, cls=None):
        """
        Same as `pdoc.Doc.__init__`, except `cls` should be provided
        as a `pdoc.Class` object when this is a class or instance
        variable.
        """
        super().__init__(name, module, docstring)

        self.cls = cls
        """
        The `podc.Class` object if this is a class or instance
        variable. If not, this is None.
        """

    @property
    def source(self):
        return []

    @property
    def refname(self):
        if self.cls is None:
            return "%s.%s" % (self.module.refname, self.name)
        else:
            return "%s.%s" % (self.cls.refname, self.name)


class External(Doc):
    """
    A representation of an external identifier. The textual
    representation is the same as an internal identifier, but without
    any context. (Usually this makes linking more difficult.)

    External identifiers are also used to represent something that is
    not exported but appears somewhere in the public interface (like
    the ancestor list of a class).
    """

    __pdoc__[
        "External.docstring"
    ] = """
        An empty string. External identifiers do not have
        docstrings.
        """
    __pdoc__[
        "External.module"
    ] = """
        Always `None`. External identifiers have no associated
        `pdoc.Module`.
        """
    __pdoc__[
        "External.name"
    ] = """
        Always equivalent to `pdoc.External.refname` since external
        identifiers are always expressed in their fully qualified
        form.
        """

    def __init__(self, name):
        """
        Initializes an external identifier with `name`, where `name`
        should be a fully qualified name.
        """
        super().__init__(name, None, "")

    @property
    def source(self):
        return []

    @property
    def refname(self):
        return self.name



"""This module defines the programmatic API that can be used to interact with `pdocs`
   to generate and view documentation from Python source code.

   If you want to extend `pdocs` or use it directly from within Python - this is the place
   to start.
"""
import os
import pathlib
import sys
import tempfile
import webbrowser

import hug

import pdocs.extract
import pdocs.logo
import pdocs.render
import pdocs.static
from pdocs import defaults


def as_html(
    modules: list,
    output_dir: str = defaults.HTML_OUTPUT_DIRECTORY,
    overwrite: bool = False,
    external_links: bool = False,
    exclude_source: bool = False,
    link_prefix: str = "",
    template_dir: str = "",
) -> str:
    """Produces HTML formatted output into the specified output_dir.

    - *modules*: One or more python module names. These may be import paths resolvable in the
      current environment, or file paths to a Python module or package.
    - *output_dir*: The directory to output HTML files to.
    - *overwrite*: If set, will overwrites any existing files in the output location.
    - *external_links*: When set, identifiers to external modules are turned into links.
    - *exclude_source*: When set, source code will not be viewable in the generated HTML.
    - *link_prefix*: A prefix to use for every link in the generated documentation otherwise
      relative links will be used.
    - *template_dir*: Specify a directory containing override Mako templates.

    Returns the `output_dir` on success.
    """
    if template_dir:
        pdocs.render.tpl_lookup.directories.insert(0, template_dir)

    roots = _get_root_modules(modules)
    destination = _destination(output_dir, roots, overwrite)
    pdocs.static.html_out(
        destination,
        roots,
        external_links=external_links,
        source=not exclude_source,
        link_prefix=link_prefix,
    )
    return output_dir


def as_markdown(
    modules: list,
    output_dir: str = defaults.MARKDOWN_OUTPUT_DIRECTORY,
    overwrite: bool = False,
    exclude_source: bool = False,
    template_dir: str = "",
) -> str:
    """Produces Markdown formatted output into the specified output_dir.

    - *modules*: One or more python module names. These may be import paths resolvable in the
      current environment, or file paths to a Python module or package.
    - *output_dir*: The directory to output HTML files to.
    - *overwrite*: If set, will overwrites any existing files in the output location.
    - *exclude_source*: When set, source code will not be viewable in the generated Markdown.
    - *template_dir*: Specify a directory containing override Mako templates.

    Returns the `output_dir` on success.
    """
    if template_dir:
        pdocs.render.tpl_lookup.directories.insert(0, template_dir)

    roots = _get_root_modules(modules)
    destination = _destination(output_dir, roots, overwrite)
    pdocs.static.md_out(destination, roots, source=not exclude_source)
    return output_dir


def server(
    modules: list,
    external_links: bool = False,
    exclude_source: bool = False,
    link_prefix: str = "",
    template_dir: str = "",
    open_browser: bool = False,
    port: int = defaults.SERVER_PORT,
    host: str = defaults.SERVER_HOST,
) -> None:
    """Runs a development webserver enabling you to browse documentation locally.

    - *modules*: One or more python module names. These may be import paths resolvable in the
      current environment, or file paths to a Python module or package.
    - *external_links*: When set, identifiers to external modules are turned into links.
    - *exclude_source*: When set, source code will not be viewable in the generated HTML.
    - *link_prefix*: A prefix to use for every link in the generated documentation otherwise
      relative links will be used.
    - *template_dir*: Specify a directory containing override Mako templates.
    - *open_browser*: If true a browser will be opened pointing at the documentation server
    - *port*: The port to expose your documentation on (defaults to: `8000`)
    - *host*: The host to expose your documentation on (defaults to `"127.0.0.1"`)
    """
    with tempfile.TemporaryDirectory() as output_dir:
        as_html(
            modules,
            overwrite=True,
            output_dir=output_dir,
            external_links=external_links,
            template_dir=template_dir,
        )

        if len(modules) == 1:
            output_dir = os.path.join(output_dir, modules[0])

        api = hug.API("Doc Server")

        @hug.static("/", api=api)
        def my_static_dirs():  # pragma: no cover
            return (output_dir,)

        @hug.startup(api=api)
        def custom_startup(*args, **kwargs):  # pragma: no cover
            print(pdocs.logo.ascii_art)
            if open_browser:
                webbrowser.open_new(f"http://{host}:{port}")

        api.http.serve(host=host, port=port, no_documentation=True, display_intro=False)


def _get_root_modules(module_names):
    if not module_names:
        sys.exit("Please provide one or more modules")
    try:
        return [pdocs.extract.extract_module(module_name) for module_name in module_names]
    except pdocs.extract.ExtractError as error:
        sys.exit(str(error))


def _destination(directory, root_modules, overwrite):
    destination = pathlib.Path(directory)
    if not overwrite and pdocs.static.would_overwrite(destination, root_modules):
        sys.exit("Rendering would overwrite files, but --overwrite is not set")
    return destination



"""
Module pdoc provides types and functions for accessing the public
documentation of a Python module. This includes modules (and
sub-modules), functions, classes and module, class and instance
variables.  Docstrings are taken from modules, functions and classes
using the special `__doc__` attribute. Docstrings for variables are
extracted by examining the module's abstract syntax tree.

The public interface of a module is determined through one of two
ways. If `__all__` is defined in the module, then all identifiers in
that list will be considered public. No other identifiers will be
considered as public. Conversely, if `__all__` is not defined, then
`pdoc` will heuristically determine the public interface. There are
three rules that are applied to each identifier in the module:

1. If the name starts with an underscore, it is **not** public.

2. If the name is defined in a different module, it is **not** public.

3. If the name refers to an immediate sub-module, then it is public.

Once documentation for a module is created with `pdoc.Module`, it
can be output as either HTML or plain text using the covenience
functions `pdoc.html` and `pdoc.text`, or the corresponding methods
`pdoc.Module.html` and `pdoc.Module.text`.

Alternatively, you may run an HTTP server with the `pdoc` script
included with this module.


Compatibility
-------------
`pdoc` requires Python 3.6 or later.


Contributing
------------
`pdoc` [is on GitHub](https://github.com/mitmproxy/pdoc). Pull
requests and bug reports are welcome.


Linking to other identifiers
----------------------------
In your documentation, you may link to other identifiers in
your module or submodules. Linking is automatically done for
you whenever you surround an identifier with a back quote
(grave). The identifier name must be fully qualified. For
example, <code>`pdoc.Doc.docstring`</code> is correct while
<code>`Doc.docstring`</code> is incorrect.

If the `pdoc` script is used to run an HTTP server, then external
linking to other packages installed is possible. No extra work is
necessary; simply use the fully qualified path. For example,
<code>`nflvid.slice`</code> will create a link to the `nflvid.slice`
function, which is **not** a part of `pdoc` at all.


Where does pdoc get documentation from?
---------------------------------------
Broadly speaking, `pdoc` gets everything you see from introspecting the
module. This includes words describing a particular module, class,
function or variable. While `pdoc` does some analysis on the source
code of a module, importing the module itself is necessary to use
Python's introspection features.

In Python, objects like modules, functions, classes and methods have
a special attribute named `__doc__` which contains that object's
*docstring*.  The docstring comes from a special placement of a string
in your source code.  For example, the following code shows how to
define a function with a docstring and access the contents of that
docstring:

    #!python
    >>> def test():
    ...     '''This is a docstring.'''
    ...     pass
    ...
    >>> test.__doc__
    'This is a docstring.'

Something similar can be done for classes and modules too. For classes,
the docstring should come on the line immediately following `class
...`. For modules, the docstring should start on the first line of
the file. These docstrings are what you see for each module, class,
function and method listed in the documentation produced by `pdoc`.

The above just about covers *standard* uses of docstrings in Python.
`pdoc` extends the above in a few important ways.


### Special docstring conventions used by `pdoc`

**Firstly**, docstrings can be inherited. Consider the following code
sample:

    #!python
    >>> class A (object):
    ...     def test():
    ...         '''Docstring for A.'''
    ...
    >>> class B (A):
    ...     def test():
    ...         pass
    ...
    >>> print(A.test.__doc__)
    Docstring for A.
    >>> print(B.test.__doc__)
    None

In Python, the docstring for `B.test` is empty, even though one was
defined in `A.test`. If `pdoc` generates documentation for the above
code, then it will automatically attach the docstring for `A.test` to
`B.test` only if `B.test` does not have a docstring. In the default
HTML output, an inherited docstring is grey.

**Secondly**, docstrings can be attached to variables, which includes
module (or global) variables, class variables and instance variables.
Python by itself [does not allow docstrings to be attached to
variables](http://www.python.org/dev/peps/pep-0224). For example:

    #!python
    variable = "SomeValue"
    '''Docstring for variable.'''

The resulting `variable` will have no `__doc__` attribute. To
compensate, `pdoc` will read the source code when it's available to
infer a connection between a variable and a docstring. The connection
is only made when an assignment statement is followed by a docstring.

Something similar is done for instance variables as well. By
convention, instance variables are initialized in a class's `__init__`
method.  Therefore, `pdoc` adheres to that convention and looks for
docstrings of variables like so:

    #!python
    def __init__(self):
        self.variable = "SomeValue"
        '''Docstring for instance variable.'''

Note that `pdoc` only considers attributes defined on `self` as
instance variables.

Class and instance variables can also have inherited docstrings.

**Thirdly and finally**, docstrings can be overridden with a special
`__pdoc__` dictionary that `pdoc` inspects if it exists. The keys of
`__pdoc__` should be identifiers within the scope of the module. (In
the case of an instance variable `self.variable` for class `A`, its
module identifier would be `A.variable`.) The values of `__pdoc__`
should be docstrings.

This particular feature is useful when there's no feasible way of
attaching a docstring to something. A good example of this is a
[namedtuple](http://goo.gl/akfXJ9):

    #!python
    __pdoc__ = {}

    Table = namedtuple('Table', ['types', 'names', 'rows'])
    __pdoc__['Table.types'] = 'Types for each column in the table.'
    __pdoc__['Table.names'] = 'The names of each column in the table.'
    __pdoc__['Table.rows'] = 'Lists corresponding to each row in the table.'

`pdoc` will then show `Table` as a class with documentation for the
`types`, `names` and `rows` members.

Note that assignments to `__pdoc__` need to placed where they'll be
executed when the module is imported. For example, at the top level
of a module or in the definition of a class.

If `__pdoc__[key] = None`, then `key` will not be included in the
public interface of the module.
"""

from pdocs._version import __version__
from pdocs.api import as_html, as_markdown, server



SERVER_PORT = 8080
SERVER_HOST = "127.0.0.1"

HTML_OUTPUT_DIRECTORY = "site"
MARKDOWN_OUTPUT_DIRECTORY = "docs"

MARKDOWN_EXTENSIONS = [
    "markdown.extensions.abbr",
    "markdown.extensions.admonition",
    "markdown.extensions.attr_list",
    "markdown.extensions.def_list",
    "markdown.extensions.fenced_code",
    "markdown.extensions.footnotes",
    "markdown.extensions.tables",
    "markdown.extensions.smarty",
    "markdown.extensions.toc",
]
MARKDOWN_EXTENSION_CONFIGS = {
    "markdown.extensions.smarty": {
        "smart_angled_quotes": False,
        "smart_dashes": True,
        "smart_quotes": False,
        "smart_ellipses": True,
    }
}



import os
import re
import sys

import markdown
import pygments
import pygments.formatters
import pygments.lexers

import pdocs.doc
import pdocs.render
from pdocs import defaults

# From language reference, but adds '.' to allow fully qualified names.
pyident = re.compile(r"^[a-zA-Z_][a-zA-Z0-9_.]+$")
indent = re.compile(r"^\s*")

_markdown = markdown.Markdown(
    output_format="html5",
    extensions=defaults.MARKDOWN_EXTENSIONS,
    extension_configs=defaults.MARKDOWN_EXTENSION_CONFIGS,
)


def _markdown_render(text):
    return _markdown.reset().convert(text)


def decode(s):
    if sys.version_info[0] < 3 and isinstance(s, str):
        return s.decode("utf-8", "ignore")
    return s


def ident(s):
    return '<span class="ident">%s</span>' % s


def sourceid(dobj):
    return "source-%s" % dobj.refname


def clean_source_lines(lines):
    """
  Cleans the source code so that pygments can render it well.

  Returns one string with all of the source code.
  """
    base_indent = len(indent.match(lines[0]).group(0))
    base_indent = 0
    for line in lines:
        if len(line.strip()) > 0:
            base_indent = len(indent.match(lines[0]).group(0))
            break
    lines = [line[base_indent:] for line in lines]

    if sys.version_info[0] < 3:
        pylex = pygments.lexers.PythonLexer()
    else:
        pylex = pygments.lexers.Python3Lexer()

    htmlform = pygments.formatters.HtmlFormatter(cssclass="codehilite")
    return pygments.highlight("".join(lines), pylex, htmlform)


def linkify(parent, match, link_prefix):
    matched = match.group(0)
    ident = matched[1:-1]
    name, url = lookup(parent, ident, link_prefix)
    if name is None:
        return matched
    return "[`%s`](%s)" % (name, url)


def mark(text, module_list=None, linky=True):
    if linky:
        text, _ = re.subn("\b\n\b", " ", text)
    return _markdown_render(text.strip())


def glimpse(s, length=100):
    if len(s) < length:
        return s
    return s[0:length] + "..."


def module_url(parent, m, link_prefix):
    """
        Returns a URL for `m`, which must be an instance of `Module`.
        Also, `m` must be a submodule of the module being documented.

        Namely, '.' import separators are replaced with '/' URL
        separators. Also, packages are translated as directories
        containing `index.html` corresponding to the `__init__` module,
        while modules are translated as regular HTML files with an
        `.m.html` suffix. (Given default values of
        `pdoc.html_module_suffix` and `pdoc.html_package_name`.)
    """
    if parent.name == m.name:
        return ""

    base = m.name.replace(".", "/")
    if len(link_prefix) == 0:
        base = os.path.relpath(base, parent.name.replace(".", "/"))
    url = base[len("../") :] if base.startswith("../") else "" if base == ".." else base
    if m.submodules:
        index = pdocs.render.html_package_name
        url = url + "/" + index if url else index
    else:
        url += pdocs.render.html_module_suffix
    return link_prefix + url


def external_url(refname):
    """
        Attempts to guess an absolute URL for the external identifier
        given.

        Note that this just returns the refname with an ".ext" suffix.
        It will be up to whatever is interpreting the URLs to map it
        to an appropriate documentation page.
    """
    return "/%s.ext" % refname


def is_external_linkable(name):
    return pyident.match(name) and "." in name


def lookup(module, refname, link_prefix):
    """
        Given a fully qualified identifier name, return its refname
        with respect to the current module and a value for a `href`
        attribute. If `refname` is not in the public interface of
        this module or its submodules, then `None` is returned for
        both return values. (Unless this module has enabled external
        linking.)

        In particular, this takes into account sub-modules and external
        identifiers. If `refname` is in the public API of the current
        module, then a local anchor link is given. If `refname` is in the
        public API of a sub-module, then a link to a different page with
        the appropriate anchor is given. Otherwise, `refname` is
        considered external and no link is used.
    """
    d = module.find_ident(refname)
    if isinstance(d, pdocs.doc.External):
        if is_external_linkable(refname):
            return d.refname, external_url(d.refname)
        else:
            return None, None
    if isinstance(d, pdocs.doc.Module):
        return d.refname, module_url(module, d, link_prefix)
    if module.is_public(d.refname):
        return d.name, "#%s" % d.refname
    return d.refname, "%s#%s" % (module_url(module, d.module, link_prefix), d.refname)


def link(parent, refname, link_prefix):
    """
        A convenience wrapper around `href` to produce the full
        `a` tag if `refname` is found. Otherwise, plain text of
        `refname` is returned.
    """
    name, url = lookup(parent, refname, link_prefix)
    if name is None:
        return refname
    return '<a href="%s">%s</a>' % (url, name)



import pathlib
import typing

import pdocs.doc
import pdocs.render


class StaticError(Exception):
    pass


def module_to_path(m: pdocs.doc.Module, extension="html") -> pathlib.Path:
    """
        Calculates the filesystem path for the static output of a given module.
    """
    p = pathlib.Path(*m.name.split("."))
    if m.submodules:
        p /= f"index.{extension}"
    else:
        p = p.with_suffix(f".{extension}")
    return p


def path_to_module(
    roots: typing.Sequence[pdocs.doc.Module], path: pathlib.Path
) -> pdocs.doc.Module:
    """
        Retrieves the matching module for a given path from a module tree.
    """
    if path.suffix == ".html":
        path = path.with_suffix("")
    parts = list(path.parts)
    if parts[-1] == "index":
        parts = parts[:-1]
    elif parts[-1] == "index.m":
        parts[-1] = "index"
    for root in roots:
        mod = root.find_ident(".".join(parts))
        if isinstance(mod, pdocs.doc.Module):
            return mod
    raise StaticError("No matching module for {path}".format(path=path))


def would_overwrite(destination: pathlib.Path, roots: typing.Sequence[pdocs.doc.Module]) -> bool:
    """Would rendering root to dst overwrite any file?"""
    if len(roots) > 1:
        path = destination / "index.html"
        if path.exists():
            return True
    for root in roots:
        if destination.joinpath(root.name).exists():
            return True
    return False


def html_out(
    dst: pathlib.Path,
    roots: typing.Sequence[pdocs.doc.Module],
    external_links: bool = True,
    link_prefix: str = "",
    source: bool = False,
):
    if len(roots) > 1:
        dst.mkdir(parents=True, exist_ok=True)
        p = dst / "index.html"
        idx = pdocs.render.html_index(roots, link_prefix=link_prefix)
        p.write_text(idx, encoding="utf-8")
    for root in roots:
        for m in root.allmodules():
            p = dst.joinpath(module_to_path(m))
            p.parent.mkdir(parents=True, exist_ok=True)
            out = pdocs.render.html_module(
                m, external_links=external_links, link_prefix=link_prefix, source=source
            )
            p.write_text(out, encoding="utf-8")


def md_out(
    dst: pathlib.Path,
    roots: typing.Sequence[pdocs.doc.Module],
    externel_links: bool = True,
    source: bool = False,
):
    for root in roots:
        for m in root.allmodules():
            p = dst.joinpath(module_to_path(m, extension="md"))
            p.parent.mkdir(parents=True, exist_ok=True)
            out = pdocs.render.text(m, source=source)
            p.write_text(out, encoding="utf-8")



import os.path
import re
import typing

from mako.exceptions import TopLevelLookupException
from mako.lookup import TemplateLookup

import pdocs.doc

html_module_suffix = ".html"
html_package_name = "index.html"
"""
The file name to use for a package's `__init__.py` module.
"""

_template_path = [os.path.join(os.path.dirname(__file__), "templates")]
"""
A list of paths to search for Mako templates used to produce the
plain text and HTML output. Each path is tried until a template is
found.
"""

tpl_lookup = TemplateLookup(
    directories=_template_path, cache_args={"cached": True, "cache_type": "memory"}
)
"""
A `mako.lookup.TemplateLookup` object that knows how to load templates
from the file system. You may add additional paths by modifying the
object's `directories` attribute.
"""


def _get_tpl(name):
    """
    Returns the Mako template with the given name. If the template cannot be
    found, a nicer error message is displayed.
    """
    try:
        t = tpl_lookup.get_template(name)
    except TopLevelLookupException:
        locs = [os.path.join(p, name.lstrip("/")) for p in _template_path]
        raise IOError(2, "No template at any of: %s" % ", ".join(locs))
    return t


def html_index(roots: typing.Sequence[pdocs.doc.Module], link_prefix: str = "/") -> str:
    """
        Render an HTML module index.
    """
    t = _get_tpl("/html_index.mako")
    t = t.render(roots=roots, link_prefix=link_prefix)
    return t.strip()


def html_module(
    mod: pdocs.doc.Module, external_links: bool = False, link_prefix: str = "/", source: bool = True
) -> str:
    """
    Returns the documentation for the module `module_name` in HTML
    format. The module must be importable.

    If `external_links` is `True`, then identifiers to external modules
    are always turned into links.

    If `link_prefix` is `True`, then all links will have that prefix.
    Otherwise, links are always relative.

    If `source` is `True`, then source code will be retrieved for
    every Python object whenever possible. This can dramatically
    decrease performance when documenting large modules.
    """
    t = _get_tpl("/html_module.mako")
    t = t.render(
        module=mod, external_links=external_links, link_prefix=link_prefix, show_source_code=source
    )
    return t.strip()


def text(mod: pdocs.doc.Module, source: bool = True) -> str:
    """Returns the documentation for the module `module_name` in plain
    text format. The module must be importable.

    *source* - If set to True (the default) source will be included in the produced output.
    """
    t = _get_tpl("/text.mako")
    text, _ = re.subn("\n\n\n+", "\n\n", t.render(module=mod, show_source_code=source).strip())
    return text



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pdocs']

package_data = \
{'': ['*'], 'pdocs': ['templates/*']}

install_requires = \
['Mako>=1.1,<2.0',
 'Markdown>=3.0.0,<4.0.0',
 'docstring_parser>=0.7.2,<0.8.0',
 'hug>=2.6,<3.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.7,<0.8']}

entry_points = \
{'console_scripts': ['pdocs = pdocs.cli:__hug__.cli']}

setup_kwargs = {
    'name': 'pdocs',
    'version': '1.1.1',
    'description': 'A simple program and library to auto generate API documentation for Python modules.',
    'long_description': "[![pdocs - Documentation Powered by Your Python Code.](https://raw.github.com/timothycrosley/pdocs/master/art/logo_large.png)](https://timothycrosley.github.io/pdocs/)\n_________________\n\n[![PyPI version](https://badge.fury.io/py/pdocs.svg)](http://badge.fury.io/py/pdocs)\n[![Build Status](https://travis-ci.org/timothycrosley/pdocs.svg?branch=master)](https://travis-ci.org/timothycrosley/pdocs)\n[![codecov](https://codecov.io/gh/timothycrosley/pdocs/branch/master/graph/badge.svg)](https://codecov.io/gh/timothycrosley/pdocs)\n[![Join the chat at https://gitter.im/pdocs/community](https://badges.gitter.im/pdocs/community.svg)](https://gitter.im/pdocs/community?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)\n[![License](https://img.shields.io/github/license/mashape/apistatus.svg)](https://pypi.python.org/pypi/pdocs/)\n[![Downloads](https://pepy.tech/badge/pdocs)](https://pepy.tech/project/pdocs)\n_________________\n\n[Read Latest Documentation](https://timothycrosley.github.io/pdocs/) - [Browse GitHub Code Repository](https://github.com/timothycrosley/pdocs/)\n_________________\n\n\n`pdocs` is a library and a command line program to discover the public\ninterface of a Python module or package. The `pdocs` script can be used to\ngenerate Markdown or HTML of a module's public interface, or it can be used\nto run an HTTP server that serves generated HTML for installed modules.\n\n`pdocs` is an MIT Licensed fork of [pdoc](https://github.com/mitmproxy/pdoc)'s original implementation by Andrew Gallant (@BurntSushi).\n with the goal of staying true to the original vision layed out by the project's creator.\n\nNOTE: For most projects, the best way to use `pdocs` is using [portray](https://timothycrosley.github.io/portray/).\n\n[![asciicast](https://asciinema.org/a/265744.svg)](https://asciinema.org/a/265744)\n\nFeatures\n--------\n\n* Support for documenting data representation by traversing the abstract syntax\n  to find docstrings for module, class and instance variables.\n* For cases where docstrings aren't appropriate (like a\n  [namedtuple](http://docs.python.org/2.7/library/collections.html#namedtuple-factory-function-for-tuples-with-named-fields)),\n  the special variable `__pdocs__` can be used in your module to\n  document any identifier in your public interface.\n* Usage is simple. Just write your documentation as Markdown. There are no\n  added special syntax rules.\n* `pdocs` respects your `__all__` variable when present.\n* `pdocs` will automatically link identifiers in your docstrings to its\n  corresponding documentation.\n* When `pdocs` is run as an HTTP server, external linking is supported between\n  packages.\n* The `pdocs` HTTP server will cache generated documentation and automatically\n  regenerate it if the source code has been updated.\n* When available, source code for modules, functions and classes can be viewed\n  in the HTML documentation.\n* Inheritance is used when possible to infer docstrings for class members.\n\nThe above features are explained in more detail in pdocs's documentation.\n\n`pdocs` is compatible with Python 3.6 and newer.\n\n## Quick Start\n\nThe following guides should get you up using pdocs in no time:\n\n1. [Installation](https://timothycrosley.github.io/pdocs/docs/quick_start/1.-installation/) - TL;DR: Run `pip3 install pdocs` within your projects virtual environment.\n2. [Command Line Usage](https://timothycrosley.github.io/pdocs/docs/quick_start/2.-cli/) - TL;DR: Run `pdocs server YOUR_MODULES` to test and `pdocs as_html YOUR_MODULES` to generate HTML.\n3. [API Usage](https://timothycrosley.github.io/pdocs/docs/quick_start/3.-api/) - TL;DR: Everything available via the CLI is also easily available programmatically from within Python.\n\n## Differences Between pdocs and pdoc\n\nBelow is a running list of intentional differences between [pdoc](https://github.com/mitmproxy/pdoc) and [pdocs](https://github.com/timothycrosley/pdocs):\n\n- pdocs has built-in support for Markdown documentation generation (as needed by [portray](https://timothycrosley.github.io/portray/)).\n- pdocs has built-in support for the inclusion of Type Annotation information in reference documentation.\n- pdocs requires Python 3.6+; pdoc maintains Python2 compatibility as of the latest public release.\n- pdocs uses the most recent development tools to ensure long-term maintainability (mypy, black, isort, flake8, bandit, ...)\n- pdocs generates project documentation to a temporary folder when serving locally, instead of including a live server. An intentional trade-off between simplicity and performance.\n- pdocs provides a simplified Python API in addition to CLI API.\n- pdocs is actively maintained.\n- pdocs uses [hug CLI and sub-commands](https://github.com/timothycrosley/pdocs/blob/master/pdocs/cli.py#L1), pdoc uses [argparse and a single command](https://github.com/mitmproxy/pdoc/blob/master/pdoc/cli.py#L1).\n- pdoc provides textual documentation from the command-line, pdocs removed this feature for API simplicity.\n\n## Notes on Licensing and Fork\n\nThe original pdoc followed the [Unlicense license](https://unlicense.org/), and as such so does the initial commit to this fork [here](https://github.com/timothycrosley/pdocs/commit/7cf925101e4ffc5690f2952ac9ad0b7b0410b4f8).\nUnlicense is fully compatible with MIT, and the reason for the switch going forward is because MIT is a more standard and well-known license.\n\nAs seen by that commit, I chose to fork with fresh history, as the project is very old (2013) and I felt many of the commits that happened in the past might, instead of helping to debug issues, lead to red herrings due to the many changes that have happened\nin the Python eco-system since that time. If you desire to see the complete history for any reason, it remains available on the original [pdoc repository](https://github.com/mitmproxy/pdoc).\n\n## Why Create `pdocs`?\n\nI created `pdocs` to help power [portray](https://timothycrosley.github.io/portray/) while staying true to the original vision of `pdoc` and maintain\nMIT license compatibility. In the end I created it to help power better documentation websites for Python projects.\n\nI hope you, too, find `pdocs` useful!\n\n~Timothy Crosley\n",
    'author': 'Timothy Crosley',
    'author_email': 'timothy.crosley@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
